#include <stdlib.h>
#include "scilabLink.h"
#include <api_scilab.h>
#include <Scierror.h>


int int32ArrayReading(int pos, int32Array *ia) {
    SciErr sciErr;
    int iPrec;
    int *piAddr     = NULL;
    sciErr = getVarAddressFromPosition(pvApiCtx, pos, &piAddr);
	if(sciErr.iErr)
	{
		printError(&sciErr, 0);
		return 0;
	}
    sciErr = getVarDimension(pvApiCtx,piAddr,&(ia->nRows),&(ia->nCols));
    if ((ia->nRows==0)&&(ia->nCols==0)) return 1;
    if(sciErr.iErr)
	{
		printError(&sciErr, 0);
		return 0;
	}
    
    sciErr = getMatrixOfIntegerPrecision(pvApiCtx, piAddr, &iPrec);
	if(sciErr.iErr || iPrec != SCI_INT32)
	{
		printError(&sciErr, 0);
		return 0;
	}
    sciErr = getMatrixOfInteger32(pvApiCtx,piAddr,&(ia->nRows),&(ia->nCols),&(ia->data));
    if(sciErr.iErr)
	{
		printError(&sciErr, 0);
		return 0;
	}
    return 1;
}
int doubleArrayReading(int pos, doubleArray *da) {
    SciErr sciErr;
    int *piAddr     = NULL;
    sciErr = getVarAddressFromPosition(pvApiCtx, pos, &piAddr);
	if(sciErr.iErr)
	{
		printError(&sciErr, 0);
		return 0;
	}
    sciErr = getMatrixOfDouble(pvApiCtx,piAddr,&(da->nRows),&(da->nCols),&(da->data));
    if(sciErr.iErr)
	{
		printError(&sciErr, 0);
		return 0;
	}
    return 1;
}
int booleanArrayReading(int pos, booleanArray *ba) {
    SciErr sciErr;
    int *piAddr     = NULL;
    sciErr = getVarAddressFromPosition(pvApiCtx, pos, &piAddr);
	if(sciErr.iErr)
	{
		printError(&sciErr, 0);
		return 0;
	}
    sciErr = getMatrixOfBoolean(pvApiCtx,piAddr,&(ba->nRows),&(ba->nCols),&(ba->data));
    if(sciErr.iErr)
	{
		printError(&sciErr, 0);
		return 0;
	}
    return 1;
}

int stringArrayReading(int pos, stringArray *sa) {
    SciErr sciErr;
    int *piAddr     = NULL;
    int* piLen		= NULL;
    int i;
    sciErr = getVarAddressFromPosition(pvApiCtx, pos, &piAddr);
	if(sciErr.iErr)
	{
		printError(&sciErr, 0);
		return 0;
	}
    sciErr = getMatrixOfString(pvApiCtx,piAddr,&(sa->nRows),&(sa->nCols),NULL,NULL);
    if(sciErr.iErr)
	{
		printError(&sciErr, 0);
		return 0;
	}
    piLen = (int*)malloc(sizeof(int) * sa->nRows * sa->nCols);
    sciErr = getMatrixOfString(pvApiCtx,piAddr,&(sa->nRows),&(sa->nCols),piLen,NULL);
    if(sciErr.iErr)
	{
		printError(&sciErr, 0);
		return 0;
	}
    sa->data = (char**)malloc(sizeof(char*) * sa->nRows * sa->nCols);
    for(i = 0 ; i < sa->nRows * sa->nCols ; i++)
	{
		sa->data[i] = (char*)malloc(sizeof(char) * (piLen[i] + 1));//+ 1 for null termination
	}
    sciErr = getMatrixOfString(pvApiCtx,piAddr,&(sa->nRows),&(sa->nCols),piLen,sa->data);
    if(sciErr.iErr)
	{
		printError(&sciErr, 0);
		return 0;
	}
    return 1;
}

int doubleArrayWriting(int pos, doubleArray *da) {
    SciErr sciErr;
    sciErr = createMatrixOfDouble(pvApiCtx,Rhs+pos,da->nRows,da->nCols,da->data);
    if(sciErr.iErr)
	{
        printError(&sciErr, 0);
		return 0;
	}
    AssignOutputVariable(pvApiCtx, 1) = nbInputArgument(pvApiCtx) + 1;
    ReturnArguments(pvApiCtx);
    return 1;
}

int int32ArrayWriting(int pos, int32Array *sa) {
    SciErr sciErr;
    /* Attention : int 32 bits sur Windows 32 bits*/
    sciErr = createMatrixOfInteger32(pvApiCtx,Rhs+pos,sa->nRows,sa->nCols,sa->data);
    if(sciErr.iErr)
	{
		printError(&sciErr, 0);
		return 0;
	}
    AssignOutputVariable(pvApiCtx, 1) = nbInputArgument(pvApiCtx) + 1;
    ReturnArguments(pvApiCtx);
    return 1;
}


int stringArrayFree(stringArray *sa) {
    int i;
    for (i=0; i < sa->nRows * sa->nCols; i++) {
        free(sa->data[i]);
    }
    free(sa->data);
}
