/*
Copyright Frédéric LEGRAND, Lycée Chateaubriand, Rennes (2013) 

Ce logiciel est un programme informatique servant d'interface entre
Scilab et les convertisseurs N/A Eurosmart SysamPCI et SysamSP5 

Ce logiciel est régi par la licence CeCILL soumise au droit français et
respectant les principes de diffusion des logiciels libres. Vous pouvez
utiliser, modifier et/ou redistribuer ce programme sous les conditions
de la licence CeCILL telle que diffusée par le CEA, le CNRS et l'INRIA 
sur le site "http://www.cecill.info".

En contrepartie de l'accessibilité au code source et des droits de copie,
de modification et de redistribution accordés par cette licence, il n'est
offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
seule une responsabilité restreinte pèse sur l'auteur du programme,  le
titulaire des droits patrimoniaux et les concédants successifs.

A cet égard  l'attention de l'utilisateur est attirée sur les risques
associés au chargement,  à l'utilisation,  à la modification et/ou au
développement et à la reproduction du logiciel par l'utilisateur étant 
donné sa spécificité de logiciel libre, qui peut le rendre complexe à 
manipuler et qui le réserve donc à des développeurs et des professionnels
avertis possédant  des  connaissances  informatiques approfondies.  Les
utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
logiciel à leurs besoins dans des conditions permettant d'assurer la
sécurité de leurs systèmes et ou de leurs données et, plus généralement, 
à l'utiliser et l'exploiter dans les mêmes conditions de sécurité. 

Le fait que vous puissiez accéder à cet en-tête signifie que vous avez 
pris connaissance de la licence CeCILL, et que vous en avez accepté les
termes.
*/
#include <stdlib.h>
#include <api_scilab.h>
#include <Scierror.h>
#include "scilabLink.h"
#include "sysam.h"

Handle handle;

void _return_integer(int v) {
    createScalarInteger8(pvApiCtx, nbInputArgument(pvApiCtx) + 1, 0);
    AssignOutputVariable(pvApiCtx, 1) = nbInputArgument(pvApiCtx) + 1;
    ReturnArguments(pvApiCtx);
}

int can_ouvrir(char *fname,unsigned long fname_len) {
    stringArray nom;
    int32Array canPresent;
    int r;
    canPresent.nRows = 1;
    canPresent.nCols = 1;
    CheckInputArgument(pvApiCtx, 1, 1);
    CheckOutputArgument(pvApiCtx, 1, 1) ;
    stringArrayReading(1,&nom);
    handle.tabEntrees = NULL;
    handle.tabTemps = NULL;
    handle.tabSortieS1 = NULL;
    handle.tabSortieS2 = NULL;
    sprintf(handle.errormsg,"");
    handle.present = 0;
    if (strcmp(nom.data[0],"SysamSP5")==0) {
        handle.can_ouvrir = sp5_ouvrir;
        handle.can_fermer = sp5_fermer;
        handle.can_config_entrees = sp5_config_entrees;
        handle.can_config_trigger = sp5_config_trigger;
        handle.can_config_echantillon = sp5_config_echantillon;
        handle.can_acquerir = sp5_acquerir;
        handle.can_acquerir_avec_sorties = sp5_acquerir_avec_sorties;
        handle.can_config_sortie = sp5_config_sortie;
        handle.can_declencher_sorties = sp5_declencher_sorties;
        handle.can_stopper_sorties = sp5_stopper_sorties;
        handle.can_activer_lecture = sp5_activer_lecture;
        handle.can_lire = sp5_lire;
        handle.can_desactiver_lecture = sp5_desactiver_lecture;
        handle.can_ecrire = sp5_ecrire;
        handle.can_lancer = sp5_lancer;
        handle.can_lancer_avec_sorties = sp5_lancer_avec_sorties;
    }
    else if (strcmp(nom.data[0],"SysamPCI")==0) {
        handle.can_ouvrir = pci_ouvrir;
        handle.can_fermer = pci_fermer;
        handle.can_config_entrees = pci_config_entrees;
        handle.can_config_trigger = pci_config_trigger;
        handle.can_config_echantillon = pci_config_echantillon;
        handle.can_acquerir = pci_acquerir;
        handle.can_acquerir_avec_sorties = pci_acquerir_avec_sorties;
        handle.can_config_sortie = pci_config_sortie;
        handle.can_declencher_sorties = pci_declencher_sorties;
        handle.can_stopper_sorties = pci_stopper_sorties;
        handle.can_activer_lecture = pci_activer_lecture;
        handle.can_lire = pci_lire;
        handle.can_desactiver_lecture = pci_desactiver_lecture;
        handle.can_ecrire = pci_ecrire;
        handle.can_lancer = pci_lancer;
        handle.can_lancer_avec_sorties = pci_lancer_avec_sorties;
    }
    else {
        stringArrayFree(&nom);
        Scierror(42,"CAN inconnu : choisir SysamSP5 ou SysamPCI\n");
        return 0;
    }
   r = handle.can_ouvrir(&handle);
   canPresent.data = &r;
   if (r) {
       sciprint("%s ouvert\n",nom.data[0]);
   }
   else {
       Scierror(42,"%s absente\n",nom.data[0]);
       return 0;
   }
   int32ArrayWriting(1,&canPresent);
   stringArrayFree(&nom);
   _return_integer(0);
   return 0;
}

int can_fermer(char *fname,unsigned long fname_len) {
    CheckInputArgument(pvApiCtx, 0, 0);
    CheckOutputArgument(pvApiCtx, 1, 1) ;
    handle.can_fermer(&handle);
    if (handle.tabEntrees!=NULL) free(handle.tabEntrees);
    if (handle.tabTemps!=NULL) free(handle.tabTemps);
    if (handle.tabSortieS1!=NULL) free(handle.tabSortieS1);
    if (handle.tabSortieS2!=NULL) free(handle.tabSortieS2);
    _return_integer(0);
    return 0;
}

int can_config_entrees(char *fname,unsigned long fname_len) {
    int32Array listeVoies;
    doubleArray listeCalibres;
    int32Array listeDifferentiel;
    CheckInputArgument(pvApiCtx, 3, 3);
    CheckOutputArgument(pvApiCtx, 1, 1) ;
    int32ArrayReading(1,&listeVoies);
    doubleArrayReading(2,&listeCalibres);
    int32ArrayReading(3,&listeDifferentiel);
    if (!handle.can_config_entrees(&handle,listeVoies.data,listeVoies.nCols,listeCalibres.data,listeCalibres.nCols,listeDifferentiel.data,listeDifferentiel.nCols)) {
        Scierror(42,"Erreur de configuration des entrées : %s\n",handle.errormsg);
        return 0;
    }
    _return_integer(0);
    return 0;
}

int can_config_trigger(char *fname,unsigned long fname_len) {
    booleanArray externe;
    int32Array voie;
    doubleArray seuil;
    booleanArray montant;
    int32Array pretrigger;
    booleanArray pretriggerSouple;
    int32Array hysteresis;
    CheckInputArgument(pvApiCtx, 7, 7);
    CheckOutputArgument(pvApiCtx, 1, 1) ;
    booleanArrayReading(1,&externe);
    int32ArrayReading(2,&voie);
    doubleArrayReading(3,&seuil);
    booleanArrayReading(4,&montant);
    int32ArrayReading(5,&pretrigger);
    booleanArrayReading(6,&pretriggerSouple);
    int32ArrayReading(7,&hysteresis);
    if (!handle.can_config_trigger(&handle,externe.data[0],voie.data[0],seuil.data[0],montant.data[0],pretrigger.data[0],pretriggerSouple.data[0],hysteresis.data[0])) {
        Scierror(42,"Erreur de configuration du trigger : %s\n",handle.errormsg);
        return 0;
    }
    _return_integer(0);
    return 0;
}



int can_config_echantillon(char *fname,unsigned long fname_len) {
    doubleArray temps;
    int32Array nbpoints;
    CheckInputArgument(pvApiCtx, 2, 2);
    CheckOutputArgument(pvApiCtx, 1, 1) ;
    doubleArrayReading(1,&temps);
    int32ArrayReading(2,&nbpoints);
    //sciprint("te : %f",temps.data[0]);
    //sciprint("nbpoints : %d",nbpoints.data[0]);
    if (!handle.can_config_echantillon(&handle,temps.data[0],nbpoints.data[0])) {
        Scierror(42,"Erreur de configuration de l'échantillonnage : %s\n",handle.errormsg);
        return 0;
    }
    _return_integer(0);
    return 0;
}
int can_acquerir(char *fname,unsigned long fname_len) {
    CheckInputArgument(pvApiCtx, 0, 0);
    CheckOutputArgument(pvApiCtx, 1, 1) ;
    if (!handle.can_acquerir(&handle)) {
        Scierror(42,"Erreur d'acquisition : %s\n",handle.errormsg);
        return 0;
    }
    _return_integer(0);
    return 0;
}
int can_lancer(char *fname,unsigned long fname_len) {
    CheckInputArgument(pvApiCtx, 0, 0);
    CheckOutputArgument(pvApiCtx, 1, 1) ;
    if (!handle.can_lancer(&handle)) {
        Scierror(42,"Erreur d'acquisition : %s\n",handle.errormsg);
        return 0;
    }
    _return_integer(0);
    return 0;   
}

int can_acquerir_avec_sorties(char *fname,unsigned long fname_len) {
    doubleArray valeurs1, valeurs2;
    CheckInputArgument(pvApiCtx, 2, 2);
    CheckOutputArgument(pvApiCtx, 1, 1) ;
    doubleArrayReading(1,&valeurs1);
    doubleArrayReading(2,&valeurs2);
    if (!handle.can_acquerir_avec_sorties(&handle,valeurs1.data,valeurs1.nCols,valeurs2.data,valeurs2.nCols)) {
        Scierror(42,"Erreur d'acquisition : %s\n",handle.errormsg);
        return 0;
    }
    _return_integer(0);
    return 0; 
}

int can_lancer_avec_sorties(char *fname,unsigned long fname_len) {
    doubleArray valeurs1, valeurs2;
    CheckInputArgument(pvApiCtx, 2, 2);
    CheckOutputArgument(pvApiCtx, 1, 1) ;
    doubleArrayReading(1,&valeurs1);
    doubleArrayReading(2,&valeurs2);
    if (!handle.can_lancer_avec_sorties(&handle,valeurs1.data,valeurs1.nCols,valeurs2.data,valeurs2.nCols)) {
        Scierror(42,"Erreur d'acquisition : %s\n",handle.errormsg);
        return 0;
    }
    _return_integer(0);
    return 0;   
} 

int can_entrees(char *fname,unsigned long fname_len) {
    doubleArray entrees;
    int nbpoints;
    CheckInputArgument(pvApiCtx, 0, 0);
    CheckOutputArgument(pvApiCtx, 1, 1) ;
    if (handle.nbpointsEAdemande < handle.nbpointsEA) nbpoints = handle.nbpointsEAdemande;
    else nbpoints = handle.nbpointsEA;
    entrees.nCols = nbpoints;
    entrees.nRows = handle.nombreEA;
    entrees.data = handle.tabEntrees;
    doubleArrayWriting(1,&entrees);
    return 0;
}

/* récupération d'un paquet contenant le temps et les entrées*/
/* premier : indice du premier point (compté à partir de 1)*/
int can_paquet(char *fname,unsigned long fname_len) {
    int32Array premierA;
    int premier;
    int n,k,p,v;
    doubleArray paquet;
    int nbpoints;
    CheckInputArgument(pvApiCtx, 1, 1);
    CheckOutputArgument(pvApiCtx, 1, 1) ;
    int32ArrayReading(1,&premierA);
    premier = premierA.data[0];
    if (premier<=0) premier = 1;
    nbpoints = handle.nbpointsEAlus;
    paquet.nCols = nbpoints-premier+1;
    paquet.nRows = handle.nombreEA*2;
    if ((paquet.nCols<=0)||(handle.tabEntrees==NULL)) {
        
           paquet.nCols = 0;
           paquet.nRows = 0;
           doubleArrayWriting(1,&paquet);
           return 0;
    }
    paquet.data = malloc(sizeof(double)*paquet.nCols*paquet.nRows);
    for (p=premier-1; p < nbpoints; p++) {
        for (v=0; v < handle.nombreEA; v++) {
            k = (p-premier+1)*handle.nombreEA*2+v;
            paquet.data[k] = handle.tabTemps[p*handle.nombreEA+v];
            paquet.data[k+handle.nombreEA] = handle.tabEntrees[p*handle.nombreEA+v];
        }
    }
    doubleArrayWriting(1,&paquet);
    free(paquet.data);
    return 0;
}

int can_temps(char *fname,unsigned long fname_len) {
    doubleArray temps;
    int nbpoints;
    CheckInputArgument(pvApiCtx, 0, 0);
    CheckOutputArgument(pvApiCtx, 1, 1) ;
    if (handle.nbpointsEAdemande < handle.nbpointsEA) nbpoints = handle.nbpointsEAdemande;
    else nbpoints = handle.nbpointsEA;
    temps.nCols = nbpoints;
    temps.nRows = handle.nombreEA;
    temps.data = handle.tabTemps;
    doubleArrayWriting(1,&temps);
    return 0;
}
int can_config_sortie(char *fname,unsigned long fname_len) {
    int32Array nsortie;
    doubleArray te;
    doubleArray valeurs;
    int32Array repetition;
    CheckInputArgument(pvApiCtx, 4, 4);
    CheckOutputArgument(pvApiCtx, 1, 1) ;
    int32ArrayReading(1,&nsortie);
    doubleArrayReading(2,&te);
    doubleArrayReading(3,&valeurs);
    int32ArrayReading(4,&repetition);
    if (!handle.can_config_sortie(&handle,nsortie.data[0],te.data[0],valeurs.data,valeurs.nCols,repetition.data[0])) {
        Scierror(42,"Erreur de configuration d'une sortie : %s\n",handle.errormsg);
        return 0;
    }
    _return_integer(0);
    return 0;
}
int can_declencher_sorties(char *fname,unsigned long fname_len) {
    int32Array ns1;
    int32Array ns2;
    CheckInputArgument(pvApiCtx, 2, 2);
    CheckOutputArgument(pvApiCtx, 1, 1) ;
    int32ArrayReading(1,&ns1);
    int32ArrayReading(2,&ns2);
    if (!handle.can_declencher_sorties(&handle,ns1.data[0],ns2.data[0])) {
        Scierror(42,"Erreur de déclenchement des sorties : %s\n",handle.errormsg);
        return 0;
    }
    _return_integer(0);
    return 0;
}
int can_stopper_sorties(char *fname,unsigned long fname_len) {
    int32Array ns1;
    int32Array ns2;
    CheckInputArgument(pvApiCtx, 2, 2);
    CheckOutputArgument(pvApiCtx, 1, 1) ;
    int32ArrayReading(1,&ns1);
    int32ArrayReading(2,&ns2);
    if (!handle.can_stopper_sorties(&handle,ns1.data[0],ns2.data[0])) {
        Scierror(42,"Erreur d'arrêt des sorties : %s\n",handle.errormsg);
        return 0;
    }
    _return_integer(0);
    return 0;
}

int can_activer_lecture(char *fname,unsigned long fname_len) {
    int32Array listeVoies;
    int32Array latch;
    CheckInputArgument(pvApiCtx, 2, 2);
    CheckOutputArgument(pvApiCtx, 1, 1) ;
    int32ArrayReading(1,&listeVoies);
    int32ArrayReading(2,&latch);
    if (!handle.can_activer_lecture(&handle,listeVoies.data,listeVoies.nCols,latch.data[0])) {
        Scierror(42,"Erreur d'activation de la lecture rapide : %s\n",handle.errormsg);
        return 0;
    }
    _return_integer(0);
    return 0;
}

int can_lire(char *fname,unsigned long fname_len) {
    doubleArray  entrees;
    CheckInputArgument(pvApiCtx, 0, 0);
    CheckOutputArgument(pvApiCtx, 1, 1) ;
    if (!handle.can_lire(&handle)) {
        Scierror(42,"Erreur de lecture : %s\n",handle.errormsg);
        return 0;
    }
    entrees.nCols = 1;
    entrees.nRows = handle.nombreEAdirect;
    entrees.data = handle.tensionsLues;
    doubleArrayWriting(1,&entrees);
    return 1;
}

int can_desactiver_lecture(char *fname,unsigned long fname_len) {
    CheckInputArgument(pvApiCtx, 0, 0);
    CheckOutputArgument(pvApiCtx, 1, 1) ;
    if (!handle.can_desactiver_lecture(&handle)) {
        Scierror(42,"Erreur de désactivation de la lecture rapide : %s\n",handle.errormsg);
        return 0;
    }
    _return_integer(0);
    return 0;
}

int can_ecrire(char *fname,unsigned long fname_len) {
    int32Array ns1;
    doubleArray value1;
    int32Array ns2;
    doubleArray value2;
    CheckInputArgument(pvApiCtx, 4, 4);
    CheckOutputArgument(pvApiCtx, 1, 1) ;
    int32ArrayReading(1,&ns1);
    doubleArrayReading(2,&value1);
    int32ArrayReading(3,&ns2);
    doubleArrayReading(4,&value2);
    if (!handle.can_ecrire(&handle,ns1.data[0],value1.data[0],ns2.data[0],value2.data[0])) {
        Scierror(42,"Erreur d'écriture sur les sorties analogiques : %s\n",handle.errormsg);
        return 0;
    }
    _return_integer(0);
    return 0;
}
