/*
Copyright Frédéric LEGRAND, Lycée Chateaubriand, Rennes (2010) 

Ce logiciel est un programme informatique servant d'interface logiciel
pour le convertisseur analogique-numérique Eurosmart SysamSP5

Ce logiciel est régi par la licence CeCILL soumise au droit français et
respectant les principes de diffusion des logiciels libres. Vous pouvez
utiliser, modifier et/ou redistribuer ce programme sous les conditions
de la licence CeCILL telle que diffusée par le CEA, le CNRS et l'INRIA 
sur le site "http://www.cecill.info". 

En contrepartie de l'accessibilité au code source et des droits de copie,
de modification et de redistribution accordés par cette licence, il n'est
offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
seule une responsabilité restreinte pèse sur l'auteur du programme,  le
titulaire des droits patrimoniaux et les concédants successifs.

A cet égard  l'attention de l'utilisateur est attirée sur les risques
associés au chargement,  à l'utilisation,  à la modification et/ou au
développement et à la reproduction du logiciel par l'utilisateur étant 
donné sa spécificité de logiciel libre, qui peut le rendre complexe à 
manipuler et qui le réserve donc à des développeurs et des professionnels
avertis possédant  des  connaissances  informatiques approfondies.  Les
utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
logiciel à leurs besoins dans des conditions permettant d'assurer la
sécurité de leurs systèmes et ou de leurs données et, plus généralement, 
à l'utiliser et l'exploiter dans les mêmes conditions de sécurité. 

Le fait que vous puissiez accéder à cet en-tête signifie que vous avez 
pris connaissance de la licence CeCILL, et que vous en avez accepté les
termes.
*/

#include <stdio.h>
#include <pthread.h>
#include "sysam.h"
#include "sysamSP5Link.h"



HINSTANCE HandleSysteme =NULL; 

Type_SP5_Reset* SP5_Reset;
Type_SP5_Present* SP5_Present;
Type_SP5_LireRegistre* SP5_LireRegistre;
Type_SP5_EcrireRegistre* SP5_EcrireRegistre;
Type_SP5_ViderFifo* SP5_ViderFifo;
Type_SP5_TailleTransfertFifo* SP5_TailleTransfertFifo;
Type_SP5_EcrireFifo* SP5_EcrireFifo;
Type_SP5_RecupereCalibration* SP5_RecupereCalibration;
Type_SP5_LireFifo* SP5_LireFifo;

TFifoSP5 FifoSP5;

/*----------------------------------------------------------------------------------------------
        Ouverture de la DLL SP5.DLL et chargement des pointeurs de fonctions
*/
int sp5_ouvrir(Handle *handle) {
    int i;
    
    if (!(HandleSysteme=LoadLibrary(TEXT("SP5.dll")))) {
        sprintf_s(handle->errormsg,MSGSIZE,"%s","Erreur : SP5.dll introuvable");
        return 0;
    }
    
    SP5_Reset=(Type_SP5_Reset*) GetProcAddress(HandleSysteme,"Reset");
	SP5_Present=(Type_SP5_Present*) GetProcAddress(HandleSysteme,"Present");
	SP5_LireRegistre=(Type_SP5_LireRegistre*) GetProcAddress(HandleSysteme,"LireRegistre");
	SP5_EcrireRegistre=(Type_SP5_EcrireRegistre*) GetProcAddress(HandleSysteme,"EcrireRegistre");
	SP5_EcrireFifo=(Type_SP5_EcrireFifo*) GetProcAddress(HandleSysteme,"EcrireFifo");
	SP5_LireFifo=(Type_SP5_LireFifo*) GetProcAddress(HandleSysteme,"LireFifo");
	SP5_ViderFifo=(Type_SP5_ViderFifo*) GetProcAddress(HandleSysteme,"ViderFifo");
	SP5_TailleTransfertFifo=(Type_SP5_TailleTransfertFifo*) GetProcAddress(HandleSysteme,"TailleTransfertFifo");
	SP5_RecupereCalibration=(Type_SP5_RecupereCalibration*) GetProcAddress(HandleSysteme,"RecupereCalibration");
    
	if (SP5_Present()) {
	    handle->tabEntrees = NULL;
	    handle->tabTemps = NULL;
	    handle->tabSortieS1 = NULL;
	    handle->tabSortieS2 = NULL;
        handle->present = 1;
        handle->nombreEA = 0;
        handle->temin = 0;
        handle->tempsEchantillon = 0;
        handle->sortieSelect[0] = 0;
        handle->sortieSelect[1] = 0;
        handle->nombreEAdirect = 0;
        handle->ramAdc = 0;
        handle->ramDac0 = 0;
        handle->ramDac1 = 0;
        for (i=0; i<8; i++) handle->calibreEA[i] = 0;
        handle->nbcalibres = 7;
        handle->maxCalibreEA[0] = 10.0; handle->maxCalibreEA[1] = 2.0;   handle->maxCalibreEA[2] = 0.2; handle->maxCalibreEA[3]=0; handle->maxCalibreEA[4] = 5; handle->maxCalibreEA[5] = 1; handle->maxCalibreEA[6] = 0.1; 
        SP5_Reset();
        SP5_EcrireRegistre(ADR_Config_Triger,0x00);
        
        return 1;
    }
    else {
        sprintf_s(handle->errormsg,MSGSIZE,"%s","Erreur : centrale sysamSP5 absente");
        return 0;
    }
}

/*----------------------------------------------------------------------------------------------
        Fermeture de la DLL
*/
int sp5_fermer(Handle *handle) {
    if (handle->acquisition==1) return 1;
    FreeLibrary(HandleSysteme);
    handle->present = 0;
    return 1;
}

/*-----------------------------------------------------------------------------------------------
        Configuration des entrées analogiques (convertisseur A/N)
            listeVoies : tableau des voies sélectionnées
            nbVoies : taille du tableau précédent
            listeCalibres : tableau des tensions maximales en volt pour les voies sélectionnées
            nbCalibres : taille du tableau précédent
            listeDiff : tableau des voies en mode différentiel
            nbDiff : taille du tableau précédent
*/
int sp5_config_entrees(Handle *handle,int *listeVoies, int nbVoies, double *listeCalibres, int nbCalibres, int *listeDiff, int nbDiff){
    int i,j,k;
    unsigned char codeVoies = 0;
    unsigned char codeDiff = 0;
    int codeCalibre = 0;
    int masque;
    if (handle->acquisition==1) return 1;
    if (handle->present==0) {
        sprintf_s(handle->errormsg,MSGSIZE,"%s","Erreur : CAN non ouvert");
        return 0;
    }
    handle->nombreEA = nbVoies;
    for (i=0; i<nbVoies; i++) {
        handle->voiesEA[i] = listeVoies[i];
        codeVoies = codeVoies | (0x1 << listeVoies[i]);
    }
    SP5_EcrireRegistre(ADR_Voie_Active,codeVoies);
    for (i=0; i<nbCalibres; i++) { // numéros des calibres sélectionnés selon la tension maximale
        handle->calibreEA[i] = 0;
        if (listeCalibres[i] > 10.0) {
            return 0;
        }
        for (j=0; j < handle->nbcalibres; j++) {
            if (handle->maxCalibreEA[j] > listeCalibres[i]) handle->calibreEA[i] = j; 
        }
    }
    for (i=0; i<nbDiff; i++) { // voies en mode différentiel
        codeDiff = codeDiff | (0x1 << listeDiff[i]);
    }
    SP5_EcrireRegistre(ADR_CAN_Differentiel,codeDiff);
    k = 0;
    for (i=0; i<8; i++) { // mise à jour de la table des voies (une seule par canal en mode différentiel)
        if ((codeVoies >> i)&0x01) {
            if (((codeVoies&16)&&(i==4)&&(codeDiff&1))||((codeVoies&32)&&(i==5)&&(codeDiff&2))||((codeVoies&64)&&(i==6)&&(codeDiff&4))||((codeVoies&128)&&(i==7)&&(codeDiff&8))) {
                // entrée secondaire 4,5,6,7 d'un canal utilisé en différentiel
                handle->nombreEA -= 1;
            }
            else {
                handle->voiesEA[k] = i;
                k += 1;
            }
        }
    }
    if (handle->nombreEA!=nbCalibres) {
        sprintf_s(handle->errormsg,MSGSIZE,"%s","Erreur : le nombre de calibres ne correspond pas au nombre de voies");
        handle->nombreEA = 0;
        return 0;
    }
    codeCalibre = 0;
    for (i=0; i < handle->nombreEA; i++) { // configuration des calibres
        masque = ~(0x7 << ((handle->voiesEA[i])*3));
        codeCalibre = (codeCalibre &masque) | (handle->calibreEA[i] << ((handle->voiesEA[i])*3));
    }
    SP5_EcrireRegistre(ADR_Voie_Gain,(unsigned char)codeCalibre&0xFF);
    SP5_EcrireRegistre(ADR_Voie_Gain+1,(unsigned char)((codeCalibre&0xFF00) >> 8));
    SP5_EcrireRegistre(ADR_Voie_Gain+2,(unsigned char)((codeCalibre&0xFF0000) >> 16));
    handle->temin = 0.1;
    if (((codeVoies&(1<<0))&&(codeVoies&(1<<4))&&((codeDiff&(1<<0))==0))||((codeVoies&(1<<1))&&(codeVoies&(1<<5))&&((codeDiff&(1<<1))==0))||((codeVoies&(1<<2))&&(codeVoies&(1<<6))&&((codeDiff&(1<<2))==0))||((codeVoies&(1<<3))&&(codeVoies&(1<<7))&&((codeDiff&(1<<3))==0))) { // deux voies d'un même canal sélectionnées en mode non différentiel
        handle->temin = 2.0; // mode multiplexé (par exemple voies 0 et 4 simultanées en mode non différentiel)
    }
    for (i=0; i < handle->nombreEA; i++) {
        handle->tdecalEA[i] = 0.0; // à revoir pour le mode différentiel ?   
    }
    return 1;
}
/*---------------------------------------------------------------------------------
        Configuration du trigger
            externe : 1 pour un déclenchement sur la voie EXT, 0 pour un déclenchement sur voie
            voie : voie de déclenchement ou -1 pour désactiver le trigger
            seuil : seuil du déclenchement
            montant : 1 pour un déclenchement sur front montant, 0 pour un front descendant
            pretrigger : nombre de points mémorisés avec le déclenchement
            pretriggerSouple : 1 si le déclenchement doit se faire si le nombre de points de pretrigger n'est pas atteint
            hysteresis : hysteresis du déclenchement
            
*/
int sp5_config_trigger(Handle *handle,int externe, int voie, double seuil, int montant, int pretrigger, int pretriggerSouple, int hysteresis){
    unsigned char codeTrigger = 0;
    int i,v;
    int codeSeuil;
    if (handle->acquisition==1) return 1;
    if (handle->present==0) {
        sprintf_s(handle->errormsg,MSGSIZE,"%s","Erreur : CAN non ouvert");
        return 0;
    }
    if (voie==-1) { // desactivation du trigger
        SP5_EcrireRegistre(ADR_Config_Triger,0x00);
        return 1;
    }
    v = 0;
    if (handle->nombreEA==0) {
        sprintf_s(handle->errormsg,MSGSIZE,"%s","Erreur : aucune voie configuree");
        return 0;   
    }
    for (i=0; i < handle->nombreEA; i++) {
        if (handle->voiesEA[i] == voie) v = 1;
    }
    if (v==0) {
        sprintf_s(handle->errormsg,MSGSIZE,"%s","Erreur : voie de trigger non configuree");
        return 0;
    }
    if (externe==0) {
        
        codeTrigger = 2|((montant&1)<<2)|((pretriggerSouple&1)<<3)|((voie&7)<<4);
        SP5_EcrireRegistre(ADR_Config_Triger,codeTrigger);
        codeSeuil = (int)((seuil/handle->maxCalibreEA[handle->calibreEA[voie]]+1.0)*2048);
        SP5_EcrireRegistre(ADR_Seuil_Triger,(unsigned char)(codeSeuil&0xFF));
        SP5_EcrireRegistre(ADR_Seuil_Triger+1,(unsigned char)((codeSeuil&0x0F00)>>8));
    }
    else {
        codeTrigger = 1|((montant&1)<<2)|((pretriggerSouple&1)<<3);
        SP5_EcrireRegistre(ADR_Config_Triger,codeTrigger);
        SP5_EcrireRegistre(ADR_Seuil_Triger,0);
        SP5_EcrireRegistre(ADR_Seuil_Triger+1,0);
    }
    SP5_EcrireRegistre(ADR_NbPoint_PreTriger,(unsigned char)(pretrigger&0xFF));
    SP5_EcrireRegistre(ADR_NbPoint_PreTriger+1,(unsigned char)((pretrigger&0xFF00)>>8));
    SP5_EcrireRegistre(ADR_NbPoint_PreTriger+2,(unsigned char)((pretrigger&0x030000)>>16));
    SP5_EcrireRegistre(ADR_Hysteresis_Triger,(unsigned char)(hysteresis));
    return 1;
}

/*--------------------------------------------------------------------------------------------------------------------
        Configuration de l'échantillonnage du convertisseur analogique->numérique
            temps : période d'échantillonnage en microsecondes
            nbpoints : nombre de points à acquérir
*/

int sp5_config_echantillon(Handle *handle, double temps, int nbpoints) {
    double nSkip;
    unsigned long int nbp;
    // nbpoints codé sur 18 bits
    unsigned char n1;
    unsigned long int n2; // le nombre de sauts d'horloge est codé sur 40 bits donc il faut un entier de 32 bits et un entier de 8 bits
    int nPaquets;
    int p;
    unsigned long int maxNpts;
    int fifo;
    if (handle->acquisition==1) return 1;
    if (handle->present==0) {
        sprintf_s(handle->errormsg,MSGSIZE,"%s","Erreur : CAN non ouvert");
        return 0;
    }
    if (handle->temin==0) {
        sprintf_s(handle->errormsg,MSGSIZE,"%s","Erreur : entrees non configurees");
        return 0;
    }
    if (temps < handle->temin) {
        sprintf_s(handle->errormsg,MSGSIZE,"%s","Erreur : periode d'echantillonnage trop faible");
        return 0;   
    }
    // nombre de sauts d'horloges (100 ns) sous forme d'un flottant 64 bits
    // la mantisse est de 52 bits, suffisant pour conversion en entier 40 bits
    nSkip = floor(temps*10.0)-1;
    if (nSkip<0.0) nSkip = 0.0;
    if (nSkip > pow(2,40)) {
       sprintf_s(handle->errormsg,MSGSIZE,"%s","Erreur : periode d'echantillonnage trop grande");
       // le maximum correspond à une période d'échantillonnage de 10^5 secondes, soit 1.27 heures
       return 0;        
    }
    // 8 bits de poids fort du nombre de sauts d'hologe
    n1 = (unsigned char) floor(nSkip/((double)0xFFFFFFFF+1.0)); // décalage à droite de 32 bits
    // 32 bits de poids faible du nombre de sauts d'hologe
    n2 = ((unsigned long )nSkip);
   // programmation du nombre de coups d'horloges entre deux lectures
    SP5_EcrireRegistre(ADR_Acquisition_Skip,(unsigned char)(n2 & 0xFF)); // poids faible
    SP5_EcrireRegistre(ADR_Acquisition_Skip+1,(unsigned char)((n2 & 0xFF00)>>8));
    SP5_EcrireRegistre(ADR_Acquisition_Skip+2,(unsigned char)((n2 & 0xFF0000)>>16));
    SP5_EcrireRegistre(ADR_Acquisition_Skip+3,(unsigned char)((n2 & 0xFF000000)>>24));
    SP5_EcrireRegistre(ADR_Acquisition_Skip+4,(unsigned char)(n1 & 0xFF));
    handle->tempsEchantillon = (n1*256+n2+1)/10.0;
    fifo = 100;
    if (handle->tempsEchantillon >= 1.0e4) fifo = 100;
    if (handle->tempsEchantillon >= 1.0e5) fifo = 10;
    handle->tailleFifo = ((int)(fifo/handle->nombreEA))*handle->nombreEA; // taille du FIFO multiple du nombre de voies
    if (handle->tempsEchantillon >= 1.0e6) handle->tailleFifo = handle->nombreEA;
    p = handle->tailleFifo / handle->nombreEA; // nombre de points (un point = n voies) dans la fifo
    nPaquets =  (int) (nbpoints*handle->nombreEA/handle->tailleFifo) ; // nombre de paquets de données de tailleFifo
    if (nPaquets*handle->tailleFifo < nbpoints*handle->nombreEA) nPaquets += 1;
    maxNpts = 0x3FFFF/handle->nombreEA; // dans le meilleur des cas, lorsque les sorties ne sont pas utilisées
    nbp = nPaquets*p;
    while (nbp > maxNpts) {
        nPaquets--;
        nbp = nPaquets*p;
    }
    handle->nbpointsEAdemande = nbpoints; // nombre de points demandés
    handle->nbpointsEA = nbp; // nombre de points à acquérir
    // programmation du nombre de points
    SP5_EcrireRegistre(ADR_Acquisition_NbPoint,(unsigned char)(nbp & 0x0000FF));
    SP5_EcrireRegistre(ADR_Acquisition_NbPoint+1,(unsigned char)((nbp & 0x00FF00)>>8));
    SP5_EcrireRegistre(ADR_Acquisition_NbPoint+2,(unsigned char)((nbp & 0xFF0000)>>16));
    handle->ramAdc = handle->nbpointsEA*handle->nombreEA;
    return 1;
}

/*-------------------------------------------------------------------------------------------------
        Acquisition complète et mémorisation du temps et des tensions lues
        La fonction retourne à la fin de l'acquisition

*/


int sp5_acquerir(Handle *handle) {
    int nData, n;
    int i,j;
    char v;
    int npts;
    int nlu;
    int reste;
    if (handle->acquisition==1) return 1;
    nData = handle->nombreEA*handle->nbpointsEA; // nombre de valeurs de 12 bits à transmettre
    if (handle->present==0) {
        sprintf_s(handle->errormsg,MSGSIZE,"%s","Erreur : CAN non ouvert");
        return 0;
    }
    if (handle->nombreEA==0) {
        sprintf_s(handle->errormsg,MSGSIZE,"%s","Erreur : entrees non configurees");
        return 0;
    }
    if (handle->tempsEchantillon == 0.0) {
        sprintf_s(handle->errormsg,MSGSIZE,"%s","Erreur : echantillonnage non configure");
        return 0;
    }
    if (handle->ramAdc+handle->ramDac0+handle->ramDac1 > 0x3FFFF) { // on vérifie que les espaces RAM ne se chevauchent pas
        sprintf_s(handle->errormsg,MSGSIZE,"%s","Erreur : memoire insuffisante, trop de points en entree et/ou en sortie");
        return 0;   
    }
    handle->nbpointsEAlus = 0;
    if (handle->tabEntrees!=NULL) free(handle->tabEntrees);
    if (handle->tabTemps!=NULL) free(handle->tabTemps);
    handle->tabEntrees = malloc(sizeof(double)*nData);
    handle->tabTemps = malloc(sizeof(double)*nData);
    SP5_TailleTransfertFifo(handle->tailleFifo);
    SP5_ViderFifo();
    SP5_EcrireRegistre(ADR_Start,SP5_LireRegistre(ADR_Start) | 0x01);
    j = 0;
    n = 0;
    npts = 0;
    handle->acquisition = 1;
    reste = nData;
    while (reste > 0) { 
        nlu = SP5_LireFifo(&FifoSP5);
        if (nlu > 0) { 
            v = 0;
            if (nlu > reste) nlu = reste;
            for (i=0; i < nlu; i++) {
                handle->tabEntrees[j+i] = (((double) (FifoSP5[i] >> 4))-2048)/2048* handle->maxCalibreEA[handle->calibreEA[handle->voiesEA[v]]];
                handle->tabTemps[j+i] = handle->tdecalEA[v]+((double)npts)*handle->tempsEchantillon*1e-6;
                v += 1;// voie suivante
                n += 1;// une donnée de 12 bits lue
                if (v == handle->nombreEA) {
                    v=0;
                    npts+=1;// un point lu (12 bits x nombre de voies)
                    handle->nbpointsEAlus += 1;
                }
            }
            j += nlu;
        }
        reste = nData-n;
    }
    SP5_EcrireRegistre(ADR_Start,SP5_LireRegistre(ADR_Start) & ~0x01);
    handle->nbpointsEA = npts;
    handle->acquisition=0;
    return 1;
}

void *start_acquisition(void *h) {
    sp5_acquerir((Handle*)h);
    pthread_exit(NULL);
    return NULL;
}

/*------------------------------------------------------------------------------------------------------
    Lancement de l'acquisition dans un fil d'exécution parallèle (Thread)

*/
int sp5_lancer(Handle *handle) {
    pthread_t thread;
    int rc;
    handle->nbpointsEAlus = 0;
    rc = pthread_create(&thread,NULL,start_acquisition,(void*)handle);
    if (rc) {
        sprintf_s(handle->errormsg,MSGSIZE,"%s","Erreur : impossible de lancer l'acquisition");
        return 0;
    }
    return 1;
}



/*--------------------------------------------------------------------------------------------------------------
        Acquisition avec utilisation synchrone des sorties
            valeurs1 : tableau des tensions à appliquer sur la sortie 1
            nbvaleurs1 : longueur du tableau précédent
            valeurs2 : tableau des tensions à appliquer sur la sortie 2
            nbvaleurs2 : longueur du tableau précédent

*/
int sp5_acquerir_avec_sorties(Handle *handle,double *valeurs1, int nbvaleurs1,double *valeurs2, int nbvaleurs2) {
    int nData, n;
    int i,j;
    char v;
    int npts;
    int ns1,ns2;
    int nlu,reste;
    nData = handle->nombreEA*handle->nbpointsEA;
    if (handle->present==0) {
        sprintf_s(handle->errormsg,MSGSIZE,"%s","Erreur : CAN non ouvert");
        return 0;
    }
    if (handle->nombreEA==0) {
        sprintf_s(handle->errormsg,MSGSIZE,"%s","Erreur : entrees non configurees");
        return 0;
    }
    if (handle->tempsEchantillon == 0.0) {
        sprintf_s(handle->errormsg,MSGSIZE,"%s","Erreur : echantillonnage non configure");
        return 0;
    }
    if (handle->tabEntrees!=NULL) free(handle->tabEntrees);
    if (handle->tabTemps!=NULL) free(handle->tabTemps);
    handle->tabEntrees = malloc(sizeof(double)*nData);
    handle->tabTemps = malloc(sizeof(double)*nData);
    // configuration des sorties
    ns1 = 0; ns2 = 0;
    
    if (nbvaleurs1) {
        if (!sp5_config_sortie(handle,1,handle->tempsEchantillon,valeurs1,nbvaleurs1,1)) return 0;
        ns1 = 1;
    }
    if (nbvaleurs2) {
        if (!sp5_config_sortie(handle,2,handle->tempsEchantillon,valeurs2,nbvaleurs2,1)) return 0;
        ns2 = 1;
    }
    if (handle->ramAdc+handle->ramDac0+handle->ramDac1 > 0x3FFFF) {
        sprintf_s(handle->errormsg,MSGSIZE,"%s","Erreur : memoire insuffisante, trop de points en entree et/ou en sortie");
        return 0;   
    }
    
    SP5_ViderFifo();
    SP5_TailleTransfertFifo(handle->tailleFifo);
    SP5_EcrireRegistre(ADR_Start,SP5_LireRegistre(ADR_Start) & ~(ns1<<1) & ~(ns2<<2));
    // declenchement simultané des entrées et des sorties
    SP5_EcrireRegistre(ADR_Start,(SP5_LireRegistre(ADR_Start)&0xF9)| (ns1<<1) | (ns2<<2) | 0x01);
    j = 0;
    n = 0;
    npts = 0;
    handle->acquisition = 1;
    handle->nbpointsEAlus = 0;
    reste = nData;
    while (reste > 0) { 
        nlu = SP5_LireFifo(&FifoSP5);
        if (nlu > 0) { 
            v = 0;
            if (nlu > reste) nlu = reste;
            for (i=0; i < nlu; i++) {
                handle->tabEntrees[j+i] = (((double) (FifoSP5[i] >> 4))-2048)/2048* handle->maxCalibreEA[handle->calibreEA[handle->voiesEA[v]]];
                handle->tabTemps[j+i] = handle->tdecalEA[v]+((double)npts)*handle->tempsEchantillon*1e-6;
                v += 1;// voie suivante
                n += 1;// une donnée de 12 bits lue
                if (v == handle->nombreEA) {
                    v=0;
                    npts+=1;// un point lu (12 bits x nombre de voies)
                    handle->nbpointsEAlus += 1;
                }
            }
            j += nlu;
        }
        reste = nData-n;
    }
    SP5_EcrireRegistre(ADR_Start,SP5_LireRegistre(ADR_Start) & ~0x01);
    SP5_EcrireRegistre(ADR_Start,SP5_LireRegistre(ADR_Start) & ~(ns1<<1) & ~(ns2<<2));
    handle->nbpointsEA = npts;
    handle->acquisition=0;
    return 1;
}

void *start_acquisition_avec_sorties(void *ha) {
    Handle* h = (Handle*) ha;
    sp5_acquerir_avec_sorties(h,h->valeursS1,h->nbpointsS1,h->valeursS2,h->nbpointsS2);
    pthread_exit(NULL);
    return NULL;
}

/*--------------------------------------------------------------------------------------------------------------
        Lancement d'une acquisition sur un fil d'exécution parallèle, avec utilisation synchrone des sorties
            valeurs1 : tableau des tensions à appliquer sur la sortie 1
            nbvaleurs1 : longueur du tableau précédent
            valeurs2 : tableau des tensions à appliquer sur la sortie 2
            nbvaleurs2 : longueur du tableau précédent

*/

int sp5_lancer_avec_sorties(Handle *handle,double *valeurs1, int nbvaleurs1,double *valeurs2, int nbvaleurs2) {
    pthread_t thread;
    int rc;
    handle->nbpointsS1 = nbvaleurs1;
    handle->valeursS1 = valeurs1;
    handle->nbpointsS2 = nbvaleurs2;
    handle->valeursS2 = valeurs2;
    rc = pthread_create(&thread,NULL,start_acquisition_avec_sorties,(void*)handle);
    if (rc) {
        sprintf_s(handle->errormsg,MSGSIZE,"%s","Erreur : impossible de lancer l'acquisition");
        return 0;
    }
    return 1;
}
/*----------------------------------------------------------------------------------------------------------------------------
        Configuration d'une sortie
            nsortie : numéro de la sortie (1 ou 2)
            te : période d'échantillonnage en microsecondes
            valeurs : tableau des tensions à appliquer en volt
            npoints : longueur du tableau précédent
            repetition : 1 pour obtenir une répétition périodique, 0 sinon
*/
int sp5_config_sortie(Handle *handle, int nsortie, double te, double *valeurs, int npoints, int repetition) {
    unsigned char adr;
    double nSkip,v;
    unsigned char n1;
    unsigned long int n2;
    int paquet, point;
    int nPaquets;
    int i,reste;
    if (handle->present==0) {
        sprintf_s(handle->errormsg,MSGSIZE,"%s","Erreur : CAN non ouvert");
        return 0;
    }
    if ((nsortie!=1)&&(nsortie!=2)) {
        sprintf_s(handle->errormsg,MSGSIZE,"%s","Erreur : numero de sortie incorrect");
        return 0;
    }
    if (te < 0.2) {
        sprintf_s(handle->errormsg,MSGSIZE,"%s","Erreur : periode d'echantillonnage de sortie trop faible (min 0.2 microsecondes)");
        return 0;
    }
    
    if (npoints > 0x1FFFF) {
        sprintf_s(handle->errormsg,MSGSIZE,"%s","Erreur : nombre de points trop grand en sortie (max 131071)");
        return 0;
    }
    if (nsortie==1) handle->ramDac0 = npoints;
    if (nsortie==2) handle->ramDac1 = npoints;
    adr = ADR_DAC0_Nbpoint+(nsortie-1)*3;
    SP5_EcrireRegistre(adr,npoints&0xFF);
    SP5_EcrireRegistre(adr+1,(npoints&0xFF00)>>8);
    SP5_EcrireRegistre(adr+2,(npoints&0x30000)>>16);
    nSkip = floor(te*5)-1;
    handle->tempsEchantillonSorties = te;
    if (nSkip < 0.0) nSkip = 0.0;
    n1 = (unsigned char) floor(nSkip/((double)0xFFFFFFFF+1.0));
    n2 = (unsigned long int) (nSkip);
    adr = ADR_DAC0_Skip+(nsortie-1)*5;
    SP5_EcrireRegistre(adr,(unsigned char)(n2 & 0xFF));
    SP5_EcrireRegistre(adr+1,(unsigned char)((n2 & 0xFF00)>>8));
    SP5_EcrireRegistre(adr+2,(unsigned char)((n2 & 0xFF0000)>>16));
    SP5_EcrireRegistre(adr+3,(unsigned char)((n2 & 0xFF000000)>>24));
    SP5_EcrireRegistre(adr+4,(unsigned char)(n1 & 0xFF));
    if (nsortie==1) {
        if (repetition==-1) {
            SP5_EcrireRegistre(ADR_DAC_MonoCoup,SP5_LireRegistre(ADR_DAC_MonoCoup)&(~0x1));
        }
        else {
            SP5_EcrireRegistre(ADR_DAC_MonoCoup,SP5_LireRegistre(ADR_DAC_MonoCoup)|0x1);
        }
        SP5_EcrireRegistre(ADR_RAM_Remplissage_Debut,0x00);
        SP5_EcrireRegistre(ADR_RAM_Remplissage_Debut+1,0x00);
        SP5_EcrireRegistre(ADR_RAM_Remplissage_Debut+2,0x00);
        SP5_EcrireRegistre(ADR_RAM_Remplissage_Fin,(npoints-1)&0xFF);
        SP5_EcrireRegistre(ADR_RAM_Remplissage_Fin+1,((npoints-1)&0xFF00)>>8);
        SP5_EcrireRegistre(ADR_RAM_Remplissage_Fin+2,((npoints-1)&0x30000)>>16); 
        
        SP5_ViderFifo();
        SP5_TailleTransfertFifo(256);
        SP5_EcrireRegistre(ADR_RAM_Remplissage_Actif,0x01);
        nPaquets = npoints/256;
        point = 0;
        for (paquet = 0; paquet < nPaquets; paquet++) {
            for (i=0; i < 256; i++) {
                v = valeurs[point];
                if (v < -10.0) v = -10.0;
                if (v > 10.0) v = 10.0;
                FifoSP5[i] = (unsigned short)floor((v+10)*204.75);
                point += 1;
            }
        SP5_EcrireFifo(&FifoSP5,256);
        }
        if (point < npoints) {
            reste = npoints-point;
            for (i=0; i < reste; i++) {
                v = valeurs[point];
                if (v < -10.0) v = -10.0;
                if (v > 10.0) v = 10.0;
                FifoSP5[i] = (unsigned short)floor((v+10)*204.75);
                point += 1;
            }
        SP5_TailleTransfertFifo(reste);
        SP5_EcrireFifo(&FifoSP5,reste);
        }
        SP5_EcrireRegistre(ADR_RAM_Remplissage_Actif,0x00);
        handle->sortieSelect[0] = 1;
    }
    else if (nsortie==2) { 
        if (repetition==-1) {
            SP5_EcrireRegistre(ADR_DAC_MonoCoup,SP5_LireRegistre(ADR_DAC_MonoCoup)&(~0x2));
        }
        else {
            SP5_EcrireRegistre(ADR_DAC_MonoCoup,SP5_LireRegistre(ADR_DAC_MonoCoup)|0x2);
        }
        SP5_EcrireRegistre(ADR_RAM_Remplissage_Debut,(0x3FFFF-npoints+1)&0xFF);
        SP5_EcrireRegistre(ADR_RAM_Remplissage_Debut+1,((0x3FFFF-npoints+1)&0xFF00)>>8);
        SP5_EcrireRegistre(ADR_RAM_Remplissage_Debut+2,((0x3FFFF-npoints+1)&0x30000)>>16);
        SP5_EcrireRegistre(ADR_RAM_Remplissage_Fin,0xFF);
        SP5_EcrireRegistre(ADR_RAM_Remplissage_Fin+1,(0xFF00)>>8);
        SP5_EcrireRegistre(ADR_RAM_Remplissage_Fin+2,(0x30000)>>16);
        
        SP5_ViderFifo();
        SP5_TailleTransfertFifo(256);
        SP5_EcrireRegistre(ADR_RAM_Remplissage_Actif,0x01);
        nPaquets = npoints/256;
        point = 0;
        for (paquet = 0; paquet < nPaquets; paquet++) {
            for (i=0; i < 256; i++) {
                FifoSP5[i] = (unsigned short)floor((valeurs[point]+10)*204.75); /* inversion du sens de remplissage A VERIFIER */ 
                point += 1;
            }
        SP5_EcrireFifo(&FifoSP5,256);
        }
        if (point < npoints) {
            reste = npoints-point;
            for (i=0; i < reste; i++) {
                FifoSP5[i] = (unsigned short)floor((valeurs[point]+10)*204.75);
                point += 1;
            }
        SP5_TailleTransfertFifo(reste);
        SP5_EcrireFifo(&FifoSP5,reste);
        }
        SP5_EcrireRegistre(ADR_RAM_Remplissage_Actif,0x00);
        handle->sortieSelect[1] = 1;
    }
    else {
        handle->tempsEchantillonSorties = 0;
        sprintf_s(handle->errormsg,MSGSIZE,"%s","Erreur : numero de sortie incorrect");
        return 0;
    }
    return 1;
}

/*----------------------------------------------------------------------------------------------------------
    Déclencher les sorties
        ns1 : 1 pour déclencher la sortie 1, 0 sinon
        ns2 : 1 pour déclencher la sortie 2, 0 sinon
*/
int sp5_declencher_sorties(Handle *handle, int ns1, int ns2) {
    if (handle->present==0) {
        sprintf_s(handle->errormsg,MSGSIZE,"%s","Erreur : CAN non ouvert");
        return 0;
    }
    if ((ns1!=1)&&(ns1!=0)&&(ns2!=1)&&(ns2!=0)) {
        sprintf_s(handle->errormsg,MSGSIZE,"%s","Erreur : l'indice d'activation doit etre 0 ou 1");
        return 0;
    }
    if ((ns1==1)&(handle->sortieSelect[0]==0)) {
        sprintf_s(handle->errormsg,MSGSIZE,"%s","Erreur : sortie 1 non configuree");
        return 0;
    }
    if ((ns2==1)&(handle->sortieSelect[1]==0)) {
        sprintf_s(handle->errormsg,MSGSIZE,"%s","Erreur : sortie 2 non configuree");
        return 0;
    }
    SP5_EcrireRegistre(ADR_Start,SP5_LireRegistre(ADR_Start) & ~(ns1<<1) & ~(ns2<<2));
    SP5_EcrireRegistre(ADR_Start,(SP5_LireRegistre(ADR_Start)&0xF9)| (ns1<<1) | (ns2<<2));
    return 1;
}

/*------------------------------------------------------------------------------------------------------------
    Stopper les sorties :
        ns1 : 1 pour stopper la sortie 1, 0 sinon
        ns2 : 1 pour stopper la sortie 2, 0 sinon

*/
int sp5_stopper_sorties(Handle *handle, int ns1, int ns2) {
    if (handle->present==0) {
        sprintf_s(handle->errormsg,MSGSIZE,"%s","Erreur : CAN non ouvert");
        return 0;
    }
    if ((ns1!=1)&&(ns1!=0)&&(ns2!=1)&&(ns2!=0)) {
        sprintf_s(handle->errormsg,MSGSIZE,"%s","Erreur : l'indice de desactivation doit etre 0 ou 1");
        return 0;
    }
    SP5_EcrireRegistre(ADR_Start,SP5_LireRegistre(ADR_Start) & ~(ns1<<1) & ~(ns2<<2));
    return 1;
}


/*-----------------------------------------------------------------------------------------------------------------
        Activer la lecture directe (sans échantillonnage) sur les entrées
            listeVoies : tableau des voies
            nbvoies : taille du tableau précédent
*/
int sp5_activer_lecture(Handle *handle, int* listeVoies, int nbVoies, int latch) {
    int mot,v;
    if (handle->present==0) {
        sprintf_s(handle->errormsg,MSGSIZE,"%s","Erreur : CAN non ouvert");
        return 0;
    }
    handle->nombreEAdirect = nbVoies;
    for (v=0; v < nbVoies; v++) handle->voiesEAdirect[v] = listeVoies[v];
    if (nbVoies==0) return 1;
    SP5_EcrireRegistre(ADR_Acquisition_Skip+4,0x00);
    SP5_EcrireRegistre(ADR_Acquisition_Skip+3,0x00);
    SP5_EcrireRegistre(ADR_Acquisition_Skip+2,0x00);
    SP5_EcrireRegistre(ADR_Acquisition_Skip+1,0x00);
    SP5_EcrireRegistre(ADR_Acquisition_Skip,0x00);
    SP5_EcrireRegistre(ADR_Start,SP5_LireRegistre(ADR_Start)| (1<<4));
    mot = SP5_LireRegistre(ADR_Start);
    if (latch==0) {
        mot = mot & ~(1<<5);
    }
    else {
        mot = mot | (1<<5);
    }
    SP5_EcrireRegistre(ADR_Start, mot);
    return 1;
}

/*------------------------------------------------------------------------------------------------
    Effectuer une lecture directe des entrées sélectionnées
*/
int sp5_lire(Handle *handle) {
    int v;
    int adr;
    unsigned int mot[16];
    double valeur;
    if (handle->present==0) {
        sprintf_s(handle->errormsg,MSGSIZE,"%s","Erreur : CAN non ouvert");
        return 0;
    }
    if (handle->nombreEAdirect == 0) {
        sprintf_s(handle->errormsg,MSGSIZE,"%s","Erreur : lecture directe non activee");
        return 0;
    }
    SP5_EcrireRegistre(ADR_Start,SP5_LireRegistre(ADR_Start)|0x01);
    for (v=0; v < handle->nombreEAdirect; v++) {
        adr = ADR_Voie0_Directe+2*handle->voiesEAdirect[v];
        mot[2*v] = SP5_LireRegistre(adr);
        mot[2*v+1] = SP5_LireRegistre(adr+1);
    }
    SP5_EcrireRegistre(ADR_Start,SP5_LireRegistre(ADR_Start)&(~0x01));
    for (v=0; v < handle->nombreEAdirect; v++) {
        valeur = (double)((mot[2*v+1]*0x100+mot[2*v])>>4);
        handle->tensionsLues[v] = (valeur-2048.0)/2048.0* handle->maxCalibreEA[handle->calibreEA[handle->voiesEAdirect[v]]];
        
    }
    return 1;
}

/*-------------------------------------------------------------------------------------------------------
        Désactiver la lecture directe
*/
int sp5_desactiver_lecture(Handle *handle){
    if (handle->present==0) {
        sprintf_s(handle->errormsg,MSGSIZE,"%s","Erreur : CAN non ouvert");
        return 0;
    }
    SP5_EcrireRegistre(ADR_Start,SP5_LireRegistre(ADR_Start) & ~0x20);
    SP5_EcrireRegistre(ADR_Start,SP5_LireRegistre(ADR_Start) & ~0x10);
    handle->nombreEAdirect = 0;
    return 1;
}


/*--------------------------------------------------------------------------------------------------------
        Ecrire une tension sur les sorties
            ns1 : 1 pour écrire la sortie 1, 0 sinon
            value1 : tension en volt à appliquer sur la sortie 1
            ns2 : 1 pour écrire la sortie 2, 0 sinon
            value2 : tension en volt à appliquer sur la sortie 2
*/
int sp5_ecrire(Handle *handle, int ns1, double value1, int ns2, double value2) {
    int mot;
    if (handle->present==0) {
        sprintf_s(handle->errormsg,MSGSIZE,"%s","Erreur : CAN non ouvert");
        return 0;
    }
    if (ns1==1) {
        if (value1 < -10.0) value1=-10.0;
        if (value1 > 10.0) value1=10.0;
        mot = (int) floor((value1+10.0)*204.75);
        SP5_EcrireRegistre(ADR_Dac0_Direct,mot&0xFF);
        SP5_EcrireRegistre(ADR_Dac0_Direct+1,(mot>>8)&0x0F);
    }
    if (ns2==1) {
        if (value2 < -10.0) value2=-10.0;
        if (value2 > 10.0) value2=10.0;
        mot = (int) floor((value2+10.0)*204.75);
        SP5_EcrireRegistre(ADR_Dac1_Direct,mot&0xFF);
        SP5_EcrireRegistre(ADR_Dac1_Direct+1,(mot>>8)&0x0F);
    }
    return 1;
}

/*-------------------------------------------------------------------------------------
           Configuration du port logique C (8 bits E/S, bits indépendants)
                bit : bit à configurer
                etat : 0 pour entrée, 1 pour sortie
*/
int sp5_portC_config(Handle *handle, unsigned char bit, unsigned char etat) {
       unsigned char masque, mot;
       masque = 1 << bit;
       if (etat) mot = SP5_LireRegistre(ADR_Port_C_OE) | masque;
       else mot = SP5_LireRegistre(ADR_Port_C_OE) &(~masque);
       SP5_EcrireRegistre(ADR_Port_C_OE,mot);
       return 1;
}

/*-------------------------------------------------------------------------------------
        Ecriture sur le port C
            bit : bit à écrire
            etat : 0 ou 1
*/
int sp5_portC_ecrire(Handle *handle, unsigned char bit, unsigned char etat) {
        unsigned char masque, mot;
       masque = 1 << bit;
       if (etat) mot = SP5_LireRegistre(ADR_Port_C) | masque;
       else mot = SP5_LireRegistre(ADR_Port_C) &(~masque);
       SP5_EcrireRegistre(ADR_Port_C,mot);
       return 1;
}

/*-------------------------------------------------------------------------------------
        Lire sur le port C
            bit : bit à lire
*/
int sp5_portC_lire(Handle *handle, unsigned char bit) {
       unsigned char mot;
       mot = SP5_LireRegistre(ADR_Port_C);
       if (mot & (1<<bit)) handle->bitC=1;
       else handle->bitC=0;
       return 1;
}
/*-------------------------------------------------------------------------------------
           Configuration du port logique B (8 bits E/S, réglage E ou S par 4 bits)
                bit : 0 pour configurer les bits 0 à 3, 1 pour les bits de 4 à 7
                etat : 0 pour entrée, 1 pour sortie
*/
int sp5_portB_config(Handle *handle, unsigned char bit, unsigned char etat) {
       unsigned char masque, mot;
       masque = 1 << bit;
       if (etat) mot = SP5_LireRegistre(ADR_Port_B_OE) | masque;
       else mot = SP5_LireRegistre(ADR_Port_B_OE) &(~masque);
       SP5_EcrireRegistre(ADR_Port_B_OE,mot);
       return 1;
}

/*-------------------------------------------------------------------------------------
        Ecriture sur le port B
            bit : bit à écrire
            etat : 0 ou 1
*/
int sp5_portB_ecrire(Handle *handle, unsigned char bit, unsigned char etat) {
        unsigned char masque, mot;
       masque = 1 << bit;
       if (etat>0) mot = SP5_LireRegistre(ADR_Port_B) | masque;
       else mot = SP5_LireRegistre(ADR_Port_B) &(~masque);
       SP5_EcrireRegistre(ADR_Port_B,mot);
       return 1;
}

/*-------------------------------------------------------------------------------------
        Lire sur le port B
            bit : bit à lire
*/
int sp5_portB_lire(Handle *handle, unsigned char bit) {
       unsigned char mot;
       mot = SP5_LireRegistre(ADR_Port_B);
       if (mot & (1<<bit)) handle->bitC=1;
       else handle->bitC=0;
       return 1;
}
